/*
 * $Id$
 *
 * Send a SIP instant message as E-Mail
 *
 * Copyright (C) 2004 Swiss Federal Institute of Technology (ETH Zurich)
 * Copyright (C) 2004 Marcel Baur <baur@ethworld.ethz.ch>
 *
 * This module is released under the GNU General Public License (GPL).
 * Visit http://www.ethworld.ethz.ch for updated information.
 *
 */

 /*
  * This file is part of ser, a free SIP server.
  * Copyright (C) 2001-2004 Fhg Fokus
  *
  * ser is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
  * For a license to use the ser software under conditions
  * other than those described here, or to purchase support for this
  * software, please contact iptel.org by e-mail at the following addresses:
  *    info@iptel.org
  *
  * ser is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

 /*
  * History:
  * --------
  * 2004-12-01 Initial version (Marcel Baur)
  */




#include "../../sr_module.h"
#include "../../action.h"
#include "../../db/db.h"
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#ifndef pid_t
#define pid_t int
#endif


static char *SILOMAIL_PIPE_CMD = "cat - | /usr/sbin/sendmail -4 -t";
static char *SILOMAIL_MAILER = "silomail mailer";
static char *SILOMAIL_URL = "http://www.my.org";

static char *TABLE_SETUP = "ethworld_setup";
static char *db_url = DEFAULT_DB_URL;

static char *FIELD_EMAIL = "email";
static char *FIELD_USERNAME = "username";
static char *FIELD_INETPHONENO = "inetPhoneNo";


MODULE_VERSION static int im_mail_f (struct sip_msg *, char *, char *);

static int mod_init (void);

/* module parameters */

static cmd_export_t cmds[] = {
  {"im_mail", im_mail_f, 0, 0, REQUEST_ROUTE},
  {0, 0, 0, 0, 0}
};

static param_export_t params[] = {
  {"pipe_cmd", STR_PARAM, &SILOMAIL_PIPE_CMD},
  {"mailer", STR_PARAM, &SILOMAIL_MAILER},
  {"url", STR_PARAM, &SILOMAIL_URL},
  {"table_setup", STR_PARAM, &TABLE_SETUP},
  {"db_url", STR_PARAM, &db_url},
  {"field_email", STR_PARAM, &FIELD_EMAIL},
  {"field_username", STR_PARAM, &FIELD_USERNAME},
  {"field_inetPhoneNo", STR_PARAM, &FIELD_INETPHONENO},
  {0, 0, 0}
};

struct module_exports exports = {
  "im_mail",
  cmds,
  params,

  mod_init,			/* module initialization function */
  0,				/* response function */
  0,				/* destroy function */
  0,				/* oncancel function */
  0				/* per-child init function */
};

#define RESULT_YES 1
#define RESULT_DROP_PACKET 0
#define RESULT_NO -1

static db_con_t *db_con;
static db_func_t *db_f;



static int
mod_init (void)
{
  int res;

  LOG (L_ERR, "silomail Module\n"
       "Copyright (C) 2004 Swiss Federal Institute of Technology (ETH Zurich)\n"
       "Copyright (C) 2004 Marcel Baur <baur@ethworld.ethz.ch>\n"
       "All rights reserved\n");

  res = bind_dbmod (db_url, db_f);
  if (res < 0)
    {
      LOG (L_ERR, "Unable to bind to db module (err=%i)\n", res);
      return RESULT_NO;
    }
  db_con = db_f->init (db_url);
  if (db_con == NULL)
    {
      LOG (L_ERR, "silomail: Error in db_init");
      return RESULT_NO;
    }
  return 0;
}

/* ------------------------------------------------------------------ */

static int
mail (const char *sender_name, const char *sender_email,
      const char *sender_inet_uri, const char *rcpt_name,
      const char *rcpt_email, const char *body)
{
  FILE *handle;
  int res;

  handle = popen (SILOMAIL_PIPE_CMD, "w");
  if (handle == NULL)
    {
      res = errno;
      printf ("silomail: Error %i opening pipe '%s'\n", res,
	      SILOMAIL_PIPE_CMD);
      perror ("fopen");
    }

  fprintf (handle, "From: \"%s\" <%s>\n", sender_name, sender_email);
  fprintf (handle, "To: \"%s\" <%s>\n", rcpt_name, rcpt_email);
  fprintf (handle, "X-Mailer: %s\n", SILOMAIL_MAILER);
  fprintf (handle, "Subject: SIP Instant Message from %s\n", sender_inet_uri);
  fprintf (handle, "X-URL: %s\n", SILOMAIL_URL);
  fprintf (handle, "\n");
  fprintf (handle, "You have received a SIP instant message from %s ",
	   sender_inet_uri);
  fprintf (handle, "<");
  fprintf (handle, "%s", sender_email);
  fprintf (handle, ">:\n");

  fprintf (handle, body);

  res = fflush (handle);
  if (res != 0)
    {
      res = errno;
      printf ("silomail: Error %i flushing pipe '%s'\n", res,
	      SILOMAIL_PIPE_CMD);
      perror ("fflush");
    }

  res = fclose (handle);
  if (res != 0)
    {
      res = errno;
      printf ("silomail: Error %i closing pipe '%s'\n", res,
	      SILOMAIL_PIPE_CMD);
      perror ("fclose");
    }

  return 0;
}



/* ---------------------------------------------------------------------------------- */

static char *
hole_to (char *body, int crop_domain)
{

  char tmp[255];
  int start, stop;
  int i, size;

  assert (body);
  size = strlen (body);
  if (size > sizeof (tmp))
    {
      size = sizeof (tmp);
    }
  assert (size > 0);

  start = 0;
  stop = 0;
  i = 0;

  if (strncmp (body, "sip:", 4) == 0)
    {
      start = 0;
    }
  else
    {
      while ((body[i] != '<') && (i < size))
	{
	  start++;
	  i++;
	}
      start++;
    }
  assert (start >= 0);

  if (start >= size)
    {
      start = 0;
    }
  assert (start < size);

  while ((body[i] != '>') && (body[i] != ';') && (body[i] != 0x0D)
	 && (i < size))
    {
      i++;
    }
  stop = i;

  if (stop >= size)
    {
      stop = size;
    }


  if ((body[start] == 's') && (body[start + 1] == 'i')
      && (body[start + 2] == 'p') && (body[start + 3] = ':'))
    {
      start += 4;
    }

  assert (stop - start > 0);

  strncpy (tmp, &body[start], stop - start);
  tmp[stop - start] = '\0';

  if (crop_domain == 1)
    {
      for (i = 0; i < stop; i++)
	{
	  if (tmp[i] == '@')
	    {
	      tmp[i] = '\0';
	    }
	}
    }

  LOG (L_ERR, "hole_to is '%s'", tmp);
  return strdup (tmp);
}



/* ================================================================ */
/* given "23123" as inet phone no, lookup user name */
/* "baurma" in mysql database */

static char *
ethz_lookup (const char *what, const char *where, const char *equals)
{
  db_key_t _k[3];
  db_val_t _v[3];
  db_key_t _c[2];
  db_res_t *res;
  int n, nc, anz, result;
  char *tmp;
  db_row_t *rows;
  db_val_t *values;

  /* SELECT */
  _c[0] = what;
  nc = 1;

  /* FROM */
  db_f->use_table (db_con, TABLE_SETUP);

  /* WHERE */
  _k[0] = where;
  n = 1;

  VAL_TYPE (&_v[0]) = DB_STR;
  VAL_NULL (&_v[0]) = 0;
  VAL_STR (&_v[0]).s = (char *) equals;
  VAL_STR (&_v[0]).len = strlen (equals);

  /* execute query */
  result = db_f->query (db_con, _k, 0, _v, _c, n, nc, 0, &res);

  if (result < 0)
    {
      LOG (L_ERR, "lookup_username_with_email: no user found\n");
      return NULL;
    }

  anz = RES_ROW_N (res);
  if (anz != 1)
    {
      return NULL;
    }
  else
    {
      /* got a result, find out which one */
      rows = RES_ROWS (res);
      values = ROW_VALUES (rows);
      tmp = VAL_STR (values).s;
      LOG (L_ERR, "user: '%s'\n", tmp);
    }
  return tmp;
}



/* ---------------------------------------------------------------------------------- */

static int
im_mail_f (struct sip_msg *mmm, char *m1, char *m2)
{
  int retval;
  char *to, *rcpt_username, *rcpt_email;
  char *from, *sender_username, *sender_email;
  char *body;

  retval = RESULT_NO;

  to = hole_to (mmm->to->body.s, 1);
  if ((to == NULL) || (to == ""))
    {
      LOG (L_ERR, "silomail: to is NULL");
      return RESULT_NO;
    }

  rcpt_username = ethz_lookup (FIELD_USERNAME, FIELD_INETPHONENO, to);
  if ((rcpt_username == NULL) || (rcpt_username == ""))
    {
      LOG (L_ERR, "silomail: rcpt_username is NULL");
      return RESULT_NO;
    }

  rcpt_email = ethz_lookup (FIELD_EMAIL, FIELD_INETPHONENO, to);
  if ((rcpt_email == NULL) || (rcpt_email == ""))
    {
      LOG (L_ERR, "silomail: rcpt_email is '%s'", rcpt_email);
      return RESULT_NO;
    }


  from = hole_to (mmm->from->body.s, 1);
  if ((from == NULL) || (from == ""))
    {
      LOG (L_ERR, "silomail: from is NULL");
      return RESULT_NO;
    }

  sender_username = ethz_lookup (FIELD_USERNAME, FIELD_INETPHONENO, from);
  if ((sender_username == NULL) || (sender_username == ""))
    {
      LOG (L_ERR, "silomail: sender_username is NULL");
      sender_username = from;
      sender_email = hole_to (mmm->from->body.s, 0);
    }
  else
    {
      sender_email = ethz_lookup (FIELD_EMAIL, FIELD_INETPHONENO, from);
      if ((sender_email == NULL) || (sender_email == ""))
	{
	  sender_email = hole_to (mmm->from->body.s, 0);
	}
    }

  body = mmm->eoh;
  if (body == NULL)
    {
      body = strdup ("No message\n");
      LOG (L_ERR, "silomail: NO MESSAGE");
    }

  retval =
    mail (sender_username, sender_email, sender_username, rcpt_username,
	  rcpt_email, body);

  LOG (L_ERR, "im_mail_f: %i", retval);
  retval = RESULT_YES;
  return retval;
}

/* ---------------------------------------------------------------------------------- */
